# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

"""
This update script is used to grant proper privileges on Subrogation Mandate files to Operator/TBS.
"""
from basesoft.web.contrib.file.models import File
from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.core.util.db import DatabaseSession
from pvcycle.core.models import PvcOperatorLicense
from pvcycle.subrogation.models import PvcSubrogation

sa_session = DatabaseSession().Session


def grant_privileges():
    c = AbstractController(sa_session)

    for subrogation in sa_session.query(PvcSubrogation).filter(PvcSubrogation.mandate_file != None):
        print 'Processing subrogation: %d' % subrogation.id
        c.grant_privilege_to(subrogation.mandate_file, File.PRIV_VIEW, license=True, target_group=subrogation.subrogator.member.default_group, check_own_privs=False)
        # grant privileges to commissioner on mandate file.
        if subrogation.commissioner:
            c.grant_privilege_to(subrogation.mandate_file, File.PRIV_VIEW, license=True, target_group=subrogation.commissioner.member.default_group, check_own_privs=False)

        # grant privileges to TBS on mandate file.
        c.grant_privilege_to(subrogation.mandate_file, File.PRIV_VIEW, license=True, target_group=subrogation.subrogator.tbs.default_group, check_own_privs=False)
        # grant privileges to Operator on mandate file.
        operators = sa_session.query(PvcOperatorLicense).filter(PvcOperatorLicense.domain==subrogation.subrogator.tbs.domain).all()
        for operator in operators:
            c.grant_privilege_to(subrogation.mandate_file, File.PRIV_VIEW, license=True, target_group=operator.default_group, check_own_privs=False)


if __name__ == "__main__":
    sa_session.begin()
    try:
        grant_privileges()
        sa_session.commit()
    except Exception, e:
        sa_session.rollback()
        raise e

