# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.license.models import Domain

from basesoft.web.core.util.db import Session
from sqlalchemy import and_, text

from pvcycle.order.configurable_widgets.declaration_result import DeclarationResultWidget
from pvcycle.order.configurable_widgets.contribution_result import ContributionResultWidget
from pvcycle.order.configurable_widgets.order_result import PvcOrderResultWidget

from pvcycle.product.models import PvcTbsProduct


def procedures():

    sa_session = Session()

    rv = [
        """
        DROP VIEW IF EXISTS `viewPvcIsFullMember`
        """,
        """
        CREATE VIEW viewPvcIsFullMember AS
        SELECT 0 as 'is_full_member', 'Non-full Member' as 'is_full_member_label'
        UNION
        SELECT 1 as 'is_full_member', 'Full Member' as 'is_full_member_label'
        """,
        """
        DROP VIEW IF EXISTS `viewPvcHasVotingRights`
        """,
        """
        CREATE VIEW viewPvcHasVotingRights AS
        SELECT 0 as 'has_voting_rights', 'No voting rights' as 'has_voting_rights_label'
        UNION
        SELECT 1 as 'has_voting_rights', 'Has voting rights' as 'has_voting_rights_label'
        """,

        """
        DROP PROCEDURE IF EXISTS `pvcPopulateTbsProductClosure`
        """,
        """
        CREATE PROCEDURE pvcPopulateTbsProductClosure()
        BEGIN
            TRUNCATE TABLE closurePvcTbsProduct;

            INSERT INTO closurePvcTbsProduct (parent_level_id, product_level_id, distance)
                SELECT parent.id, child.id, closure.distance
                FROM tblPvcTbsProduct as parent
                INNER JOIN closureWEEEProductLevel as closure ON closure.parent_level_id = parent.product_id
                INNER JOIN tblPvcTbsProduct as child ON closure.product_level_id = child.product_id;

        END
        """,

        """
        DROP VIEW IF EXISTS `viewPvcTbsMember`
        """,
        """
        CREATE VIEW viewPvcTbsMember AS
        SELECT
            tm.id AS id,
            tm.tbs_id AS tbs_id,
            tm.member_id AS member_id,
            tm.is_commissioner,
            tm.is_producer,
            tbs.name AS tbs_name,
            member.name AS member_name
        FROM tblPvcTbsMember AS tm
        INNER JOIN tblLicense AS tbs
            ON tm.tbs_id=tbs.id
        INNER JOIN tblLicense AS member
            ON tm.member_id=member.id
        """,

        """
        DROP VIEW IF EXISTS `viewPvcTbsProduct`
        """,
        """
        CREATE VIEW viewPvcTbsProduct AS
            select p.id as product_id, p.label as product_label, p.tbs_id, l.name as tbs_name
            from tblPvcTbsProduct p
            inner join tblLicense l on p.tbs_id = l.id
        """

    ]

    pvc_domain = sa_session.query(Domain).filter(Domain.name == 'PV CYCLE').first()
    if pvc_domain:
        rv.append("""
        DROP VIEW IF EXISTS `viewPvcWeeeCategory`
        """)
        rv.append("""
        CREATE VIEW viewPvcWeeeCategory AS
            SELECT p.id, p.label, p.position
            FROM tblWEEEProductLevel p
                     INNER JOIN tblWEEEProductLevelGroup pg on p.id = pg.product_level_id
                     INNER JOIN tblGroup g on pg.group_id = g.id
                     INNER JOIN tblLicense l on g.license_id = l.id
                     INNER JOIN tblDomain d on l.domain_id = d.id
            WHERE pg.world_has_privilege = 1
               OR l.domain_id = %s
            GROUP BY p.id
        """ % pvc_domain.id)

    return rv

def post_sync(sa_session, app_name):
    if app_name != 'pvcycle.order':
        return

    # FIMXE
    # sa_session.execute(text('CALL pvcPopulateTbsProductClosure').execution_options(autocommit=True))


configurable_widgets = (DeclarationResultWidget, PvcOrderResultWidget, ContributionResultWidget)
