'''
Created on Jul 23, 2014

@author: rado
'''

from basesoft.util.ordered_dict import OrderedDict

from basesoft import logging
from basesoft.web.core.util.controllers import AbstractController
from iqportal.iqube.core.reporters import ConfigurableWidgetReporter
from iqportal.iqube.xmla.mdx import MdxProvider
from pvcycle.core.models import PvcOperatorLicense, PvcTbsLicense, PvcMemberLicense

logger = logging.getLogger("iqportal.iqube.core.reporters")


class PvcWidgetReporter(ConfigurableWidgetReporter):

    def modify_widget_config(self, config):
        ac = AbstractController(self.sa_session, account_id=self.account_id)
        current_license = ac.get_default_group().license

        if isinstance(current_license, PvcOperatorLicense):
            config = self._hide_dimension(config, 'TBS Member')
            config = self._hide_dimension(config, 'TBS Producer')
        elif isinstance(current_license, PvcTbsLicense):
            config = self._hide_dimension(config, 'TBS')
            config = self._hide_dimension(config, 'Member')
            config = self._hide_dimension(config, 'Producer')
            config = self._hide_dimension(config, 'WEEE Category')
            pass
        elif isinstance(current_license, PvcMemberLicense):
            config = self._hide_dimension(config, 'TBS Member')
            config = self._hide_dimension(config, 'TBS Producer')
            config = self._hide_dimension(config, 'WEEE Category')

        return config

class PvcAbstractDeclarationReporter(PvcWidgetReporter):
    """
    The PVC Abstract Declaration reporter is used for the Top/BottomCount of members...
    """

    def get_mdx_config(self, config, applied_config):

        assert applied_config['count_opts'] in ('TopCount', 'BottomCount', 'NoSelection', '', )
        assert applied_config['select_axis'] in ('rows', 'columns', '', )

        measure_index = applied_config['dimensions']['Measures']
        measure = '[Measures].[Amount (excl. vat)]'
        # look for the first measure in the selection.
        if len(applied_config['dimension_%s' % measure_index]) > 0:
            measure = applied_config['dimension_%s' % measure_index][0]['member_unique_name']

        rv = super(PvcAbstractDeclarationReporter, self).get_mdx_config(config, applied_config)
        if applied_config['count_opts'] != 'NoSelection' and applied_config['count_opts'] != '':
            c = rv[None]['mdx_config']
            hierarchize = c['hierarchize']
            axis = applied_config['select_axis']
            formula = applied_config['count_opts']
            axis_type = 0
            if axis == 'rows':
                axis_type = 1
            elif axis == 'columns':
                axis_type = 2
            result, union = MdxProvider().process_crossjoin(c[axis], axis_type, hierarchize, process_union=False)
            c['mdx_with'].append("SET [Static Count] AS %s(%s, %s, %s)" % (formula, result, applied_config.get('number_of_members'), measure))
            c[axis] = [{"hierarchy_unique_name": '', "member_unique_name": '[Static Count]'}]

        return OrderedDict(rv)


class PvcDeclarationReporter(PvcAbstractDeclarationReporter):
    pass


class PvcContributionReporter(PvcAbstractDeclarationReporter):
    pass

class PvcOrderResultReporter(PvcWidgetReporter):
    pass

reporters = (PvcDeclarationReporter, PvcContributionReporter, PvcOrderResultReporter)
