# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime, date

import xlwt, os
from django.conf import settings
from django.utils.translation import ugettext as _
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils import simplejson
from django.http import HttpResponse

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
from basesoft.web.core import L10n_settings
from basesoft.web.core.util.controllers import PermissionController

from pvcycle.order.controllers import PvcContributionOrderController


def view_annual_confirmation(request, params):
    params = simplejson.loads(params)

    account_configs = {}

    sa_session = request.Session()
    if request.session.has_key('user_id'):
        pc = PermissionController(sa_session, account_id=request.session['user_id'])
        account_configs = pc.config.get_configs()

    settings = L10n_settings.get(request.LANGUAGE_CODE, account_configs).get_settings()

    return render_to_response('view_annual_confirmation.html', {
        'params': params,
        'signature': _('Signature'),
        'L10n': simplejson.dumps(settings, ensure_ascii=False, cls=JsonRpcJSONEncoder),
        'base_requires': ['jquery_waitforimages', 'pvc_certificate_scss', 'pvc_annual_confirmation_details', 'pvc_annual_confirmation_details_scss'],
        'theme_css': 'pvc-certificate pvc-annual-confirmation'
    }, context_instance=RequestContext(request))


def download_annual_confirmation_data_xls(request, params):
    params = simplejson.loads(params)

    sa_session = request.Session()
    cc = PvcContributionOrderController(sa_session, account_id=request.session['user_id'])
    rv = cc.get_annual_confirmation(params['id'], params['year'])

    name = 'annual_confirmation_%d_%d_%s.xls' % (params['id'], params['year'], date.strftime(datetime.now(), '%d-%m-%y_%H-%I'))
    loc = os.path.join(settings.TMP_FILE_REPOSITORY, name)

    workbook = xlwt.Workbook(encoding="UTF-8")
    sheet = workbook.add_sheet('data')

    headers = {
        'quarter': _('Quarter'),
        'month': _('Month'),
        'country': _('Country'),
        'product': _('Product'),
        'brand': _('Brand'),
        'is_b2b': _('B2B/B2C'),
        'quantity': _('Quantity'),
        'weight': _('Weight'),
        'custom_unit': _('Custom unit')
    }
    h = ['quarter', 'month', 'country', 'product', 'brand', 'is_b2b', 'quantity', 'weight', 'custom_unit']
    if not rv['has_brands']:
        del headers['brand']
    if not rv['has_end_user_distinction']:
        del headers['is_b2b']
    if rv['tbs']['address_data']['country_name'] is not None:
        del headers['country']
    # add headers to the sheet
    i = 0
    cols = {}
    for key in h:
        if not headers.has_key(key):
            continue
        cols[key] = i
        sheet.write(0, i, headers[key])
        i += 1

    j = 1
    for row in rv['details']:
        for col, val in row.iteritems():
            if not headers.has_key(col):
                continue
            sheet.write(j, cols[col], val)
        j += 1

    workbook.save(loc)
    fp = open(loc, 'rb')
    response = HttpResponse(fp, mimetype='application/excel')
    response['Content-Disposition'] = 'attachment; filename=%s' % name

    os.unlink(loc)  # cleanup the xls
    return response